# UU module
module UU

  # Operation Registry is a service for keeping records of resources assigned
  # to and consumed by territories for running uuApps.
  module OperationRegistry

    # Client of service providing operations for self-service deployment of uuApps.
    # @private
    class BaseMethods

      # Remote service path. Must be specified in descendants of this class.
      # The value must look like this: +uu-operation-registry/Cloud+.
      PATH = ""

      # Default URLs where common commands are running.
      # @private
      DEFAULT_COMMON_BASE_URL = /https:\/\/(api|cmd)\.plus4u\.net/

      # Default URL for C3 (and other syscalls).
      # @private
      DEFAULT_C3_BASE_URL = "https://sys.plus4u.net"

      attr_accessor :client
      protected :client

      def initialize(session = nil)
        self.class._init_command_client(self.class::PATH)
        @client = UU::OS::CMD::CommandClient.new(self.class::PATH, session)
      end

      def create(target_uri, parameters = {})
        client.invoke(:create, target_uri, parameters: parameters)
      end

      def self.create(target_uri, parameters = {})
        self.new(UU::OS::Security::Session.current_session).create(target_uri, parameters)
      end

      def get_attributes(target_uri)
        client.invoke(:getAttributes, target_uri)
      end

      def self.get_attributes(target_uri)
        self.new(UU::OS::Security::Session.current_session).get_attributes(target_uri)
      end

      def set_attributes(target_uri, parameters = {})
        client.invoke(:setAttributes, target_uri, parameters: parameters)
      end

      def self.set_attributes(target_uri, parameters = {})
        self.new(UU::OS::Security::Session.current_session).set_attributes(target_uri, parameters)
      end

      def delete(target_uri)
        client.invoke(:delete, target_uri)
      end

      def self.delete(target_uri)
        self.new(UU::OS::Security::Session.current_session).delete(target_uri)
      end

      # @private
      def self._init_command_client(path)
        # Try to auto-configure client base URL.
        ext_config = UU::OS::CMD::CommandClient.const_get(:EXTERNAL_CONFIGURATION)
        base_url = ext_config.get(:base_url, path)
        if base_url.nil? || base_url =~ DEFAULT_COMMON_BASE_URL
          UU::OS::CMD::CommandClient.load_configuration({"uu.operation.registry.base_url" => DEFAULT_C3_BASE_URL})
        end
        # Command is not yet invoked through uuGateway, therefore do not append routing data.
        UU::OS::CMD::CommandClient.load_configuration({"uu.operation.registry.append_routing_data" => "false"})
      end
    end

    # Predefined string values.
    # @private
    class Constants
    end
  end
end
